@@ -1,4 +1,5 @@
 use {
+    crate::sysvar_cache::SysvarCache,
     itertools::Itertools,
     solana_sdk::{
         account::AccountSharedData,
@@ -8,7 +9,7 @@ use {
         pubkey::Pubkey,
         sysvar::Sysvar,
     },
-    std::{cell::RefCell, collections::HashSet, fmt::Debug, rc::Rc, sync::Arc},
+    std::{borrow::Cow, cell::RefCell, collections::HashSet, fmt::Debug, rc::Rc, sync::Arc},
 };
 
 /// Prototype of a native loader entry point
@@ -104,8 +105,8 @@ pub trait InvokeContext {
         execute_us: u64,
         deserialize_us: u64,
     );
-    /// Get sysvar data
-    fn get_sysvar_data(&self, id: &Pubkey) -> Option<Rc<Vec<u8>>>;
+    /// Get sysvar cache
+    fn get_sysvar_cache(&self) -> &SysvarCache;
     /// Set the return data
     fn set_return_data(&mut self, return_data: Option<(Pubkey, Vec<u8>)>);
     /// Get the return data
@@ -148,15 +149,20 @@ pub fn get_sysvar<T: Sysvar>(
     invoke_context: &dyn InvokeContext,
     id: &Pubkey,
 ) -> Result<T, InstructionError> {
-    let sysvar_data = invoke_context.get_sysvar_data(id).ok_or_else(|| {
-        ic_msg!(invoke_context, "Unable to get sysvar {}", id);
-        InstructionError::UnsupportedSysvar
-    })?;
-
-    bincode::deserialize(&sysvar_data).map_err(|err| {
-        ic_msg!(invoke_context, "Unable to get sysvar {}: {:?}", id, err);
-        InstructionError::UnsupportedSysvar
-    })
+    invoke_context
+        .get_sysvar_cache()
+        .iter()
+        .find_map(|(key, data)| {
+            if id == key {
+                bincode::deserialize(data).ok()
+            } else {
+                None
+            }
+        })
+        .ok_or_else(|| {
+            ic_msg!(invoke_context, "Unable to get sysvar {}", id);
+            InstructionError::UnsupportedSysvar
+        })
 }
 
 #[derive(Clone, Copy, Debug, AbiExample, PartialEq)]
@@ -389,7 +395,7 @@ pub struct MockInvokeContext<'a> {
     pub compute_meter: MockComputeMeter,
     pub programs: Vec<(Pubkey, ProcessInstructionWithContext)>,
     pub accounts: Vec<(Pubkey, Rc<RefCell<AccountSharedData>>)>,
-    pub sysvars: Vec<(Pubkey, Option<Rc<Vec<u8>>>)>,
+    pub sysvar_cache: Cow<'a, SysvarCache>,
     pub disabled_features: HashSet<Pubkey>,
     pub return_data: Option<(Pubkey, Vec<u8>)>,
     pub execute_timings: ExecuteDetailsTimings,
@@ -407,7 +413,7 @@ impl<'a> MockInvokeContext<'a> {
             },
             programs: vec![],
             accounts: vec![],
-            sysvars: vec![],
+            sysvar_cache: Cow::Owned(SysvarCache::default()),
             disabled_features: HashSet::default(),
             return_data: None,
             execute_timings: ExecuteDetailsTimings::default(),
@@ -422,21 +428,6 @@ impl<'a> MockInvokeContext<'a> {
     }
 }
 
-pub fn mock_set_sysvar<T: Sysvar>(
-    mock_invoke_context: &mut MockInvokeContext,
-    id: Pubkey,
-    sysvar: T,
-) -> Result<(), InstructionError> {
-    let mut data = Vec::with_capacity(T::size_of());
-
-    bincode::serialize_into(&mut data, &sysvar).map_err(|err| {
-        ic_msg!(mock_invoke_context, "Unable to serialize sysvar: {:?}", err);
-        InstructionError::GenericError
-    })?;
-    mock_invoke_context.sysvars.push((id, Some(Rc::new(data))));
-    Ok(())
-}
-
 impl<'a> InvokeContext for MockInvokeContext<'a> {
     fn push(
         &mut self,
@@ -524,10 +515,8 @@ impl<'a> InvokeContext for MockInvokeContext<'a> {
         _deserialize_us: u64,
     ) {
     }
-    fn get_sysvar_data(&self, id: &Pubkey) -> Option<Rc<Vec<u8>>> {
-        self.sysvars
-            .iter()
-            .find_map(|(key, sysvar)| if id == key { sysvar.clone() } else { None })
+    fn get_sysvar_cache(&self) -> &SysvarCache {
+        &self.sysvar_cache
     }
     fn set_return_data(&mut self, return_data: Option<(Pubkey, Vec<u8>)>) {
         self.return_data = return_data;
